from typing import Optional, List, Dict
from faker import Faker
from google.cloud import dlp_v2, firestore
from .config import DEFAULT_INFO_TYPES
from .exceptions import AnonymizationError, DeAnonymizationError
from .core._fast_ops import FastStringOps


class ReversibleAnonymizer:
    def __init__(
            self,
            project: str,
            info_types: Optional[List[Dict[str, str]]] = None,
            collection_name: str = "mappings"
    ):
        self.project = project
        self.info_types = info_types or DEFAULT_INFO_TYPES
        self.collection_name = collection_name

        # Initialize clients
        self.dlp = dlp_v2.DlpServiceClient()
        self.fake = Faker()
        self.db = firestore.Client()

        # Initialize fast string operations
        self._fast_ops = FastStringOps()

    def anonymize(self, text_to_deidentify: str) -> str:
        try:
            parent = f"projects/{self.project}/locations/global"

            # Configure DLP inspection
            inspect_config = dlp_v2.InspectConfig(
                info_types=self.info_types,
                include_quote=True
            )
            item = dlp_v2.ContentItem(value=text_to_deidentify)

            # Inspect content
            response = self.dlp.inspect_content(
                request={
                    "parent": parent,
                    "inspect_config": inspect_config,
                    "item": item
                }
            )

            # Collect all replacements
            replacements = {}
            for finding in response.result.findings:
                # Check existing mappings
                docs = (
                    self.db.collection(self.collection_name)
                    .where("original_data", "==", finding.quote)
                    .stream()
                )
                docs = list(docs)

                if docs:
                    fake_data = docs[0].id
                else:
                    fake_data = self._generate_fake_data(finding.info_type.name)
                    doc_ref = self.db.collection(self.collection_name).document(fake_data)
                    doc_ref.set({"original_data": finding.quote})

                replacements[finding.quote] = fake_data

            # Use fast C++ implementation for batch replacement
            return self._fast_ops.batch_replace(text_to_deidentify, replacements)

        except Exception as e:
            raise AnonymizationError(f"Anonymization failed: {str(e)}")

    def deanonymize(self, text: str) -> str:
        try:
            # Collect all replacements
            replacements = {}
            docs = self.db.collection(self.collection_name).stream()
            for doc in docs:
                replacements[doc.id] = doc.to_dict()["original_data"]

            # Use fast C++ implementation for batch replacement
            return self._fast_ops.batch_replace(text, replacements)

        except Exception as e:
            raise DeAnonymizationError(f"De-anonymization failed: {str(e)}")